/*
 * Created on Apr 18, 2003
 */
package CH.ifa.draw.framework;

import CH.ifa.draw.io.IFAFileFilter;
import CH.ifa.draw.io.SimpleFileFilter;

import de.renew.plugin.PluginManager;
import de.renew.plugin.PluginProperties;

import java.util.Enumeration;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Set;


/**
 * @author Lawrence Cabac
 */
public class DrawingTypeManager {
    private static DrawingTypeManager dtm;
    private Hashtable drawingTypes;
    private SimpleFileFilter defaultFileFilter;
    private Set listeners;

    private DrawingTypeManager() {
        drawingTypes = new Hashtable();
        listeners = new HashSet();
    }

    public static DrawingTypeManager getInstance() {
        //this should be made more save fix!
        //i.e. the existence of the Class should be checked 
        //and maybe whether this implements Diagram (possible?)
        if (dtm == null) {
            dtm = new DrawingTypeManager();
            IFAFileFilter ifa = new IFAFileFilter();
            dtm.register("CH.ifa.draw.standard.StandardDrawing", ifa);
            dtm.setDefaultFileFilter(ifa);
            return dtm;
        }
        return dtm;
    }

    public Object register(String drawingName, SimpleFileFilter ff) {
        if (PluginProperties.debug()) {
            System.out.println("DrawingTypeManager: registering " + ff
                               + " for drawing type " + drawingName);
        }
        Object previous = drawingTypes.put(drawingName, ff);
        for (Iterator listener = listeners.iterator(); listener.hasNext();) {
            ((DrawingTypeManagerListener) listener.next()).typeRegistered(drawingName,
                                                                          ff);
        }

        return previous;
    }

    static public Drawing getDrawingFromName(String name) {
        Drawing d = null;

        //System.out.println("The name of the Drawing dtm "+name);
        try {
            d = (Drawing) Class.forName(name, true,
                                        PluginManager.getInstance()
                                                     .getBottomClassLoader())
                               .newInstance();
        } catch (InstantiationException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
        }
        return d;
    }

    public static Drawing getDrawingForFilter(SimpleFileFilter ff) {
        Hashtable hash = getInstance().getDrawingTypes();
        Enumeration enum = hash.keys();
        while (enum.hasMoreElements()) {
            String key = (String) enum.nextElement();
            SimpleFileFilter fileFilter = (SimpleFileFilter) hash.get(key);
            if (fileFilter.equals(ff)) {
                return getDrawingFromName(key);
            }
        }
        return null;
    }

    public Hashtable getDrawingTypes() {
        return drawingTypes;
    }

    //    public void setDrawings(Hashtable tab) {
    //        drawingNames = tab;
    //    }
    public boolean contains(String name) {
        return drawingTypes.containsKey(name);
    }

    public SimpleFileFilter getFilter(String name) {
        return (SimpleFileFilter) drawingTypes.get(name);

    }

    public SimpleFileFilter getDefaultFileFilter() {
        return defaultFileFilter;
    }

    public void setDefaultFileFilter(SimpleFileFilter filter) {
        if (!drawingTypes.contains(filter)) {
            throw new IllegalArgumentException("Channot choose unknown drawing type as default: "
                                               + filter);
        }
        if (PluginProperties.debug()) {
            System.out.println("DrawingTypeManager: choosing " + filter
                               + " as default drawing type.");
        }
        defaultFileFilter = filter;
        for (Iterator listener = listeners.iterator(); listener.hasNext();) {
            ((DrawingTypeManagerListener) listener.next()).defaultTypeChanged(filter);
        }
    }

    public void addListener(DrawingTypeManagerListener listener) {
        listeners.add(listener);
    }

    public void removeListener(DrawingTypeManagerListener listener) {
        listeners.remove(listener);
    }
}