package CH.ifa.draw.io.exportFormats;

import CH.ifa.draw.framework.Drawing;

import java.io.File;

import javax.swing.filechooser.FileFilter;


/**
 * This interface must be implemented to define new ExportFormats.
 */
public interface ExportFormat {

    /**
      * Returns a name for the ExportFormat, e.g. PNML.
      * @ensure result != null.
      * @return String, the Name of the ExportFormat.
      */
    public String formatName();


    /**
      * Returns the FileFilter for the ExportFormat.
      * @ensure result != null.
      * @return FileFilter, the FileFilter for the ExportFormat.
      */
    public FileFilter fileFilter();

    /**
      * Converts a drawing into a file (1 to 1 export).
      * @require path != null.
      * @require drawing != null.
      * @ensure result != null.
      * @throws Exception, is catched by the ExportHolder.
      * @param drawings drawing to be exported.
      * @param path
      * @return File, !!!!!!!!!!!!!!!!! MUSS NOCH !!!!!!!!!!!!!!!!.
      */
    public File export(Drawing drawing, File file) throws Exception;

    /**
      * Converts an array of drawings into a file (n to 1 export).
      * @require path != null.
      * @require drawings != null.
      * @require drawings.length > 0
      * @require canExportNto1()
      * @ensure result != null.
      * @throws Exception, is catched by the ExportHolder.
      * @param drawings array of drawings to be exported.
      * @param path
      * @return File, !!!!!!!!!!!!!!!!! MUSS NOCH !!!!!!!!!!!!!!!!.
      */
    public File export(Drawing[] drawings, File file) throws Exception;

    /**
      * Converts an array of drawings into an array of files (n to n export).
      * @require path != null.
      * @require path.lentgh > 0
      * @require drawings != null.
      * @require drawings.length > 0
      * @require drawings.length == path.length
      * @ensure result != null.
      * @throws Exception, is catched by the ExportHolder.
      * @param drawings array of drawings to be exported.
      * @param path
      * @return File[], !!!!!!!!!!!!!!!!! MUSS NOCH !!!!!!!!!!!!!!!!.
      */
    public File[] exportAll(Drawing[] drawings, File[] files)
            throws Exception;

    /**
     *
     * @return boolean
     */
    public boolean canExportNto1();

    /**
     * @require path != null
     * @ensure result != null
     * @param path
     * @return
     */
    public ExportFormat[] canExport(File path);

    public boolean canExportDrawing(Drawing drawing);

    /**
     *
     * @return int, -1 no shortcut
     */
    public int getShortCut();
}