/*
 * @(#)ToolButton.java 5.1
 *
 */
package CH.ifa.draw.standard;

import CH.ifa.draw.framework.Tool;

import CH.ifa.draw.util.PaletteListener;

import java.awt.Dimension;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import java.io.File;

import java.net.MalformedURLException;
import java.net.URL;

import javax.swing.AbstractButton;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JToggleButton;


/**
 * A PaletteButton that is associated with a tool.
 * @see Tool
 */
public class ToolButton implements MouseListener {
    private String fName;
    private Tool fTool;
    private JToggleButton fButton;
    private PaletteListener fListener;

    public ToolButton(PaletteListener listener, String iconName, String name,
                      Tool tool) {
        this(listener, createIcon(iconName + "1.gif"),
             createIcon(iconName + "2.gif"), name, tool);
    }

    public ToolButton(PaletteListener listener, Icon icon, Icon selectedIcon,
                      String name, Tool tool) {
        fButton = new JToggleButton();
        fButton.setIcon(icon);
        fButton.setSelectedIcon(selectedIcon);

        //		Icon bIcon = fButton.getIcon();
        fButton.setMinimumSize(new Dimension(icon.getIconWidth() + 2,
                                             icon.getIconHeight() + 2));
        fButton.addMouseListener(this);
        fTool = tool;
        fName = name;
        fListener = listener;
    }

    public Tool tool() {
        return fTool;
    }

    public String name() {
        return fName;
    }

    public Object attributeValue() {
        return tool();
    }

    public AbstractButton button() {
        return fButton;
    }

    public void mouseClicked(MouseEvent e) {
        boolean doubleClick = (e.getClickCount() > 1);
        fListener.paletteUserSelected(this, doubleClick);
    }

    public void mouseEntered(MouseEvent e) {
        fListener.paletteUserOver(this, true);
    }

    public void mouseExited(MouseEvent e) {
        fListener.paletteUserOver(this, false);
    }

    public void mousePressed(MouseEvent e) {
    }

    public void mouseReleased(MouseEvent e) {
    }

    private static ImageIcon createIcon(String name) {
        //        System.out.println("icon " + name + " "
        //                           + System.getProperty("user.dir"));
        URL url = ToolButton.class.getResource(name);
        if (url == null) {
            File icon = new File(name);
            if (!icon.isFile()) {
                return null;
            }
            try {
                url = icon.toURL();
            } catch (MalformedURLException e) {
                return null;
            }
        }
        return new ImageIcon(url);
    }
}