/*
 * @(#)SetFSRenderModeCommand.java 5.1
 *
 */
package de.renew.gui.fs;

import CH.ifa.draw.DrawPlugin;

import CH.ifa.draw.application.DrawApplication;

import CH.ifa.draw.framework.DrawingView;
import CH.ifa.draw.framework.Figure;
import CH.ifa.draw.framework.FigureEnumeration;

import CH.ifa.draw.standard.CompositeFigure;
import CH.ifa.draw.standard.StandardDrawing;

import CH.ifa.draw.util.Command;

import java.util.Enumeration;


/**
 * A command to set the mode in which Feature Structures are rendered,
 * either to "normal" or to "UML".
 * <p>
 * Clears the undo history for all drawings!
 * </p>
 */
public class SetFSRenderModeCommand extends Command {
    // protected DrawApplication fEditor;
    private boolean fUML;

    /**
     * Constructs a toggle grid command.
     * @param name the command name
     * @param editor the editor
     * @param renderUML whether UML rendering should be used
     */
    public SetFSRenderModeCommand(String name, boolean renderUML) {
        super(name);
        // getEditor() = (DrawApplication) editor;
        fUML = renderUML;
    }

    protected DrawApplication getEditor() {
        return DrawPlugin.getGui();
    }

    public void execute() {
        FSFigure.umlMode = fUML;
        // Now mark dirty all FSFigures in all drawings:
        Enumeration drawings = getEditor().drawings();
        while (drawings.hasMoreElements()) {
            StandardDrawing drawing = (StandardDrawing) drawings.nextElement();
            changeFSFigures(drawing);
            DrawingView view = getEditor().getView(drawing);
            if (view != null) {
                view.clearSelection();
                view.checkDamage();
            }


            // To avoid inconsistencies: 
            getEditor().getUndoRedoManager().clearUndoHistory(drawing);
        }
    }

    public boolean isExecutable() {
        return DrawPlugin.getGui() != null;
    }

    private void changeFSFigures(CompositeFigure figure) {
        FigureEnumeration figs = figure.figures();
        while (figs.hasMoreElements()) {
            Figure fig = figs.nextFigure();
            if (fig instanceof FSFigure) {
                fig.willChange();
                ((FSFigure) fig).markDirty();
                fig.changed();
            } else if (fig instanceof CompositeFigure) {
                changeFSFigures((CompositeFigure) fig);
            }
        }
    }
}