package de.renew.gui;

import CH.ifa.draw.util.Fontkit;

import de.renew.engine.searcher.ConsoleTracer;
import de.renew.engine.searchqueue.SearchQueue;
import de.renew.engine.searchqueue.TimeListener;

import de.renew.remote.BindingAccessor;
import de.renew.remote.RemoteTriggerable;
import de.renew.remote.TracerAccessorImpl;
import de.renew.remote.TransitionInstanceAccessor;
import de.renew.remote.TriggerableForwarder;
import de.renew.remote.TriggerableForwarderImpl;

import de.renew.util.StringUtil;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import java.rmi.RemoteException;

import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;


class BindingSelectionFrame implements ListSelectionListener, ActionListener,
                                       TimeListener, RemoteTriggerable {
    // Accesses to this field should use class synchronisation.
    private static BindingSelectionFrame instance = null;
    private JFrame frame;
    private JTextArea currentElement;
    private JList bindingList;
    private DefaultListModel bindingListModel;
    private JButton fireButton;
    private BindingAccessor[] bindings;
    private String[] fullBindingText;
    private TransitionInstanceAccessor transitionInstance;
    private CPNSimulation simulation;
    private TriggerableForwarder triggerableForwarder;

    //    private RenewMode mode;
    private boolean dirty = false;
    private boolean inSearch = false;
    private boolean wantClose = false;
    private Object updateSynchronizer = new Object();

    private BindingSelectionFrame() {
        frame = new JFrame();

        currentElement = new JTextArea(5, 20);
        currentElement.setPreferredSize(new Dimension(150, 100));
        currentElement.setEditable(false);
        currentElement.setFont(Fontkit.getFont("Monospaced", Font.PLAIN, 11));
        frame.getContentPane().add(currentElement, "Center");

        bindingListModel = new DefaultListModel();
        bindingList = new JList(bindingListModel);
        bindingList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        bindingList.addListSelectionListener(this);
        bindingList.addMouseListener(new MouseAdapter() {
                public void mouseClicked(MouseEvent e) {
                    if (e.getClickCount() >= 2) {
                        int i = bindingList.getSelectedIndex();
                        fireBinding(i);
                    }
                }
            });
        bindingList.setVisibleRowCount(10);
        JScrollPane sp = new JScrollPane(bindingList);
        sp.setPreferredSize(new Dimension(150, 100));
        frame.getContentPane().add(sp, "West");

        fireButton = new JButton(" Fire ");
        fireButton.addActionListener(this);

        JButton updateButton = new JButton(" Update ");

        updateButton.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    update();
                }
            });

        JButton closeButton = new JButton(" Close ");

        closeButton.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    close();
                }
            });

        JPanel buttonPanel = new JPanel();

        buttonPanel.add(fireButton);
        buttonPanel.add(updateButton);
        buttonPanel.add(closeButton);
        frame.getContentPane().add(buttonPanel, "South");
        frame.addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent e) {
                    close();
                }
            });
        frame.pack();


        // The binding frame will remain a listener forever.
        // It just does not evaluate the notifications, if it is closed.
        SearchQueue.insertTimeListener(this);

        try {
            triggerableForwarder = new TriggerableForwarderImpl(this);
        } catch (RemoteException e) {
            e.printStackTrace(System.err);
            JOptionPane.showMessageDialog(frame,
                                          "A problem occurred: " + e + "\n"
                                          + "See the console for details.",
                                          "Binding selection frame",
                                          JOptionPane.ERROR_MESSAGE);
        }
    }

    static synchronized void open(TransitionInstanceAccessor transitionInstance,
                                  CPNSimulation simulation) {
        if (instance == null) {
            instance = new BindingSelectionFrame();
        }
        instance.setup(transitionInstance, simulation);
    }

    static synchronized void close() {
        if (instance != null) {
            instance.instanceClose();
        }
    }

    private synchronized void instanceClose() {
        if (inSearch) {
            wantClose = true;
        } else {
            closeNow();
        }
    }

    private void closeNow() {
        // The forwarder can still be useful when the
        // BindingSelectionFrame instance is reopened.
        // But we have to deregister at the transition
        // instance accessor.
        assert triggerableForwarder != null : "Triggerable forwarder should have the same lifespan as the frame.";
        if (transitionInstance != null) {
            try {
                transitionInstance.forgetBindings(triggerableForwarder);
            } catch (RemoteException e) {
                e.printStackTrace();
                JOptionPane.showMessageDialog(frame,
                                              "Could not disconnect from transition instance:\n"
                                              + e + "\n"
                                              + "See the console for details.",
                                              "Binding selection frame",
                                              JOptionPane.WARNING_MESSAGE);
            }
        }
        frame.setVisible(false);
        bindingListModel.clear();
        transitionInstance = null;
        wantClose = false;
    }

    public void valueChanged(ListSelectionEvent e) {
        String text = "";

        int index = bindingList.getSelectedIndex();
        if (index >= 0 && index < bindings.length) {
            text = fullBindingText[index];
        }
        currentElement.setText(text);
        currentElement.setCaretPosition(0);
    }

    public void actionPerformed(ActionEvent event) {
        int index = bindingList.getSelectedIndex();
        fireBinding(index);
    }

    private void fireBinding(int index) {
        if (index >= 0 && index < bindings.length) {
            simulation.getBreakpointManager().clearLog();
            try {
                bindings[index].execute(new TracerAccessorImpl(new ConsoleTracer("Synchronously",
                                                                                 "(Manually) ")),
                                        !simulation.isStrictlySequential());
            } catch (RemoteException e) {
                e.printStackTrace(System.err);
                JOptionPane.showMessageDialog(frame,
                                              "A problem occurred: " + e + "\n"
                                              + "See the console for details.",
                                              "Binding selection frame",
                                              JOptionPane.ERROR_MESSAGE);
            }
            simulation.simulationRefresh();
            update();
        } else {
            fireButton.setEnabled(false);
        }
    }

    private void setup(TransitionInstanceAccessor transitionInstance,
                       CPNSimulation simulation) {
        this.transitionInstance = transitionInstance;
        wantClose = false;
        this.simulation = simulation;

        try {
            frame.setTitle(transitionInstance.asString()
                           + "'s possible bindings");
        } catch (RemoteException e) {
            e.printStackTrace(System.err);
            JOptionPane.showMessageDialog(frame,
                                          "A problem occurred: " + e + "\n"
                                          + "See the console for details.",
                                          "Binding selection frame",
                                          JOptionPane.ERROR_MESSAGE);
        }

        update();
    }

    /**
     * This event is issued whenever the corresponding TransitionInstance
     * may have become enabled or disabled. The triggerable has to check
     * enabledness itself.
     */
    public void proposeSearch() {
        update();
    }

    /**
     * This event is issued whenever the global simulation time
     * changes. The triggerable has to check enabledness, because
     * it only keeps track of currently activated bindings.
     */
    public void timeAdvanced() {
        update();
    }

    /**
     * Updates the binding list thread-safely.
     * This is done in a separate thread, because of
     * a deadlock problem when executing synchronously.
     */
    public void update() {
        new BindingListUpdater().start();
    }

    /**
     * Updates the binding list thread-safely.
     */
    private class BindingListUpdater extends Thread {

        /**
         * Creates a new updater.
         */
        private BindingListUpdater() {
        }

        /**
         * Updates the binding list thread-safely.
         * This method is encapsulated in a separate thread, because of
         * a deadlock problem when executing synchronously.
         */
        public void run() {
            synchronized (updateSynchronizer) {
                if (inSearch) {
                    // In a different thread there is a search prozess running.
                    // Notify it about possible new bindings and forget about it.
                    dirty = true;
                    return;
                } else if (wantClose) {
                    closeNow();
                    return;
                } else {
                    // Request the start of a new search process.
                    dirty = false;
                    inSearch = true;
                }
            }


            // When I reach this point, I have aquired the inSearch flag.
            // I am supposed to do updates until the dirty flag remains
            // false.
            while (true) {
                // Was the window already closed?
                if (transitionInstance != null) {
                    update();
                    frame.setVisible(true);
                }
                synchronized (updateSynchronizer) {
                    if (wantClose) {
                        // Close the frame and make sure that the search terminates.
                        closeNow();
                        dirty = false;
                    }
                    if (!dirty) {
                        // No more requests. Free the inSearch lock and
                        // finally exit from this method.
                        inSearch = false;
                        return;
                    } else {
                        // Start another round.
                        dirty = false;
                    }
                }
            }
        }

        /**
         * Does the AWT operations themselves.
         */
        public void update() {
            try {
                bindings = transitionInstance.findAllBindings(triggerableForwarder);
            } catch (RemoteException e) {
                e.printStackTrace(System.err);
                JOptionPane.showMessageDialog(frame,
                                              "A problem occurred: " + e + "\n"
                                              + "See the console for details.",
                                              "Binding selection frame",
                                              JOptionPane.ERROR_MESSAGE);
                bindings = new BindingAccessor[0];
            }

            fullBindingText = new String[bindings.length];
            bindingListModel.clear();
            for (int i = 0; i < bindings.length; ++i) {
                try {
                    fullBindingText[i] = bindings[i].getDescription();
                } catch (RemoteException e) {
                    e.printStackTrace(System.err);
                    JOptionPane.showMessageDialog(frame,
                                                  "A problem occurred: " + e
                                                  + "\n"
                                                  + "See the console for details.",
                                                  "Binding selection frame",
                                                  JOptionPane.ERROR_MESSAGE);
                    fullBindingText[i] = e.toString();
                }

                String oneLineText = StringUtil.unspace(fullBindingText[i]);
                if (oneLineText.length() > 50) {
                    oneLineText = oneLineText.substring(0, 47) + "...";
                }
                bindingListModel.addElement(oneLineText);
            }
            String text = "";

            if (bindings.length > 0) {
                bindingList.setSelectedIndex(0);
                text = fullBindingText[0];
            }
            currentElement.setText(text);
            currentElement.setCaretPosition(0);

            fireButton.setEnabled(bindings.length > 0);
        }
    }
}