package de.renew.gui;

import CH.ifa.draw.framework.Drawing;

import CH.ifa.draw.io.DrawingFileHelper;
import CH.ifa.draw.io.StatusDisplayer;

import de.renew.io.RNWFileFilter;

import de.renew.shadow.ShadowCompilerFactory;
import de.renew.shadow.ShadowNetSystem;

import java.io.File;
import java.io.FileOutputStream;
import java.io.ObjectOutput;
import java.io.ObjectOutputStream;

import java.util.Vector;


/**
 * Converts Renew drawings (<code>.rnw</code>) to shadow nets
 * (<code>.sns</code>).
 * <p>
 * </p>
 * ShadowTranslator.java
 * Created: Mon Jan 21  2002
 * @author Michael Duvigneau
 **/
public class ShadowTranslator {
    private static final StatusDisplayer sd = new StatusDisplayer() {
        public void showStatus(String string) {
            System.err.println(string);
        }
    };

    /**
     * Execution entry point from the command line.
     **/
    public static void main(String[] args) {
        System.setProperty("de.renew.noGraphics", "true");

        if (args.length == 0) {
            System.err.println("Usage: java de.renew.gui.ShadowTranslator file.rnw ...");
            System.err.println("Converts Renew drawings (.rnw) to shadow nets (.sns).");
            System.err.println("No syntax check or compilation is done, just conversion.");
            System.exit(1);
        }

        RNWFileFilter ff = new RNWFileFilter();
        Vector names = new Vector(args.length);
        for (int i = 0; i < args.length; ++i) {
            names.add(DrawingFileHelper.checkAndAddExtension(new File(args[i]),
                                                             ff));
        }
        CPNDrawing[] drawings = readDrawings((File[]) names.toArray(new File[names
                                                                             .size()]));
        writeShadows(drawings);
        System.exit(0);
    }

    public static CPNDrawing[] readDrawings(File[] files) {
        Vector drawings = new Vector(files.length);
        for (int i = 0; i < files.length; i++) {
            try {
                Drawing drawing = DrawingFileHelper.loadDrawing(files[i], sd);
                if (drawing == null) {
                    // Probably there already has been an error message...
                } else if (drawing instanceof CPNDrawing) {
                    drawings.add(drawing);
                } else {
                    System.out.println("  Not a CPN drawing - ignored.");
                }
            } catch (Exception e) {
                System.out.println(e);
            }
        }
        return (CPNDrawing[]) drawings.toArray(new CPNDrawing[drawings.size()]);
    }

    public static void writeShadows(CPNDrawing[] drawings) {
        writeShadows(null, drawings);
    }

    public static void writeShadows(ShadowCompilerFactory compilerFactory,
                                    CPNDrawing[] drawings) {
        int count = 0;

        // Build new shadows of all nets, one net per net system.
        for (int i = 0; i < drawings.length; i++) {
            CPNDrawing currentDrawing = drawings[i];
            ShadowNetSystem netSystem = new ShadowNetSystem(compilerFactory);
            currentDrawing.buildShadow(netSystem);

            String name = currentDrawing.getName();
            File path = currentDrawing.getFilename();
            if (path == null) {
                System.out.println("Skipping net " + name + " (no filename).");
            } else {
                try {
                    path = path.getCanonicalFile();
                    path = new File(path.getParent(), name + ".sns");
                    System.out.print("Exporting net " + name + " to " + path
                                     + " ... ");

                    FileOutputStream stream = new FileOutputStream(path);
                    ObjectOutput output = new ObjectOutputStream(stream);
                    output.writeObject(netSystem);
                    output.close();
                    System.out.println("OK");
                    count++;
                } catch (Exception e) {
                    System.out.println(e);
                }
            }
            currentDrawing.discardShadow();
        }
        System.out.println("Exported " + count + " shadow net"
                           + (count == 1 ? "" : "s") + ".");
    }
}