/*
 * (c) Copyright 2001 MyCorporation.
 * All Rights Reserved.
 */
package de.renew.plugin;



/**
 * This is the interface that all active Plugins (i.e., the mainClass
 * attribute in the plugin.cfg) must implement.
 * The init() method will be called on activating the Plugin,
 * the cleanup() method on shutting down.
 * The getProperties() must return a PluginProperties object.
 *
 * The plugins will be loaded by Implementors of the PluginLoader interface.
 * If you want to write a plugin, its constructor will probably be the
 * deciding instance by which PluginLoaders it can be instantiated.
 *
 * The standard PluginLoader, SimplePluginLoader, creates Plugin objects
 * by calling a constructor (via reflection)
 * with a PluginProperties object as its only argument:
 *
 * public IPluginImpl(PluginProperties prop) {...}
 */
public interface IPlugin {

    /**
     * Return the name of this plugin.
     */
    public String getName();

    /**
     * This method is called when the plugin is activated.
     * At this point, all requirements (given in the plugin.cfg)
     * are fulfilled and the respective classes loaded.
     */
    public void init();

    /**
     * This method is called when the plugin is shut down.
     */
    public boolean cleanup();

    /**
     * Return the PluginProperties of this plugin.
     */
    public PluginProperties getProperties();

    /**
     * Checks if the cleanup method may be called
     */
    public boolean canShutDown();
}