package de.renew.plugin.command;

import de.renew.plugin.CollectionLister;
import de.renew.plugin.IPlugin;
import de.renew.plugin.PluginManager;
import de.renew.plugin.PluginProperties;

import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;


/**
 * This class shows the properties of a previously chosen plugin.
 *
 * @author J&ouml;rn Schumacher
 */
public class InfoCommand implements CLCommand {

    /**
     * @see de.renew.plugin.command.CLCommand#execute(String[])
     */
    public void execute(String[] args) {
        // get Plugin name from input
        String pluginName = CollectionLister.toString(args, " ");
        IPlugin found = PluginManager.getInstance().getPluginByName(pluginName);
        if (found == null) {
            if (PluginProperties.debug()) {
                System.out.println("InfoCommand: did not find Plugin with name "
                                   + pluginName);
            }
            return;
        }
        try {
            PluginProperties props = found.getProperties();
            System.out.println(renderProperties(props));
        } catch (ArrayIndexOutOfBoundsException e) {
        } catch (NumberFormatException e) {
            System.out.println("cleanup canceled.");
        } catch (Exception e) {
            System.out.println("cleanup canceled: " + e + "; " + e.getMessage());
            e.printStackTrace();
        }
    }

    private String renderProperties(PluginProperties props) {
        StringBuffer result = new StringBuffer();
        result.append("Properties of plugin ");
        result.append(props.getName());
        result.append("\n\tloaded from ");
        result.append(props.getURL());
        result.append("\n\t");
        String mainClass = props.getMainClass();
        if ("".equals(mainClass)) {
            result.append("no main class");
        } else {
            result.append("main class is ");
            result.append(mainClass);
        }
        result.append("\n\tprovides ");
        result.append(CollectionLister.toString(props.getProvisions()));
        result.append("\n\trequires ");
        result.append(CollectionLister.toString(props.getRequirements()));
        result.append("\n");
        List pNames = new Vector(props.keySet());
        StringBuffer other = new StringBuffer();
        Collections.sort(pNames);
        Iterator propNames = pNames.iterator();
        while (propNames.hasNext()) {
            String currentProp = (String) propNames.next();
            if (props.isKnownProperty(currentProp)) {
                other.append("\t");
                other.append(currentProp);
                other.append("=");
                other.append(props.getProperty(currentProp));
                other.append("\n");
            }
        }
        if (other.length() > 0) {
            result.append("Other properties:\n");
            result.append(other);
        }

        return result.toString();
    }

    /**
     * @see de.renew.plugin.command.CLCommand#getDescription()
     */
    public String getDescription() {
        return "view properties of a loaded plugin";
    }
}