package de.renew.plugin.command;

import de.renew.plugin.IPlugin;
import de.renew.plugin.PluginManager;
import de.renew.plugin.PluginProperties;

import java.util.Iterator;
import java.util.Properties;
import java.util.StringTokenizer;


/**
 * This command sets a system property.
 *
 * @author J&ouml;rn Schumacher
 */
public class SetPropertyCommand implements CLCommand {

    /**
     * @see de.renew.plugin.command.CLCommand#execute(String[])
     */
    public void execute(String[] args) {
        for (int i = 0; i < args.length; i++) {
            setProperty(args[i]);
        }
    }

    protected void setProperty(String arg) {
        StringTokenizer tokenizer = new StringTokenizer(arg, "=");
        String key;
        String value = null;
        if (!tokenizer.hasMoreTokens()) {
            System.err.println("Usage: set property=value [property=value ...]");
            System.err.println("       Don't use spaces around the = sign!");
            return;
        }
        key = tokenizer.nextToken();
        if (!tokenizer.hasMoreTokens()) {
            System.out.println("SetPropertyCommand: no value given for variable "
                               + arg + ", it becomes unset.");
        } else {
            while (tokenizer.hasMoreTokens()) {
                if (value == null) {
                    value = tokenizer.nextToken();
                } else {
                    value += "=" + tokenizer.nextToken();
                }
            }
        }

        // set the property in the user properties
        Properties userProps = PluginProperties.getUserProperties();
        if (value == null) {
            userProps.remove(key);
            // iterate through all plugins and remove their property
            Iterator plugins = PluginManager.getInstance().getPlugins()
                                            .iterator();
            while (plugins.hasNext()) {
                IPlugin p = (IPlugin) plugins.next();
                PluginProperties props = p.getProperties();
                props.remove(key);
            }
        } else {
            userProps.setProperty(key, value);
            // iterate through all plugins and set their property
            Iterator plugins = PluginManager.getInstance().getPlugins()
                                            .iterator();
            while (plugins.hasNext()) {
                IPlugin p = (IPlugin) plugins.next();
                PluginProperties props = p.getProperties();
                props.setProperty(key, value);
            }
        }
    }

    /**
     * @see de.renew.plugin.command.CLCommand#getDescription()
     */
    public String getDescription() {
        return "sets a system properties (syntax key=value)";
    }
}