package de.renew.plugin.locate;

import de.renew.plugin.CollectionLister;
import de.renew.plugin.PluginProperties;

import java.io.File;
import java.io.FilenameFilter;
import java.io.InputStream;

import java.net.MalformedURLException;
import java.net.URL;

import java.util.Collection;
import java.util.Vector;
import java.util.jar.JarFile;
import java.util.zip.ZipEntry;


/**
 * This class is used to find the jar files in a directory
 * that contain a "plugin.cfg" entry.
 *
 * This is used to create the resulting PluginProperties object.
 *
 */
public class PluginJarLocationFinder extends PluginFileFinder {
    public PluginJarLocationFinder(URL basedir) {
        super(basedir);
    }

    protected File[] getPluginFiles(File dir) {
        File[] plugins = dir.listFiles(new FilenameFilter() {
                public boolean accept(File file, String name) {
                    return name.endsWith("jar");
                }
            });
        return plugins;
    }

    protected Collection getPluginConfigurations(File[] plugins) {
        // convert to urls so URLClassloader can be used
        Vector pluginURLv = new Vector();
        for (int i = 0; i < plugins.length; i++) {
            PluginProperties cfg = createPluginConfig(plugins[i]);
            if (cfg != null) {
                pluginURLv.add(cfg);
            }
        }
        return pluginURLv;
    }

    protected PluginProperties createPluginConfig(File loc) {
        if (!loc.exists()) {
            if (PluginProperties.debug()) {
                System.out.println("jar file " + loc + " does not exist.");
            }
            return null;
        }
        if (PluginProperties.debug()) {
            System.out.println("looking for plugin.cfg in jar file " + loc
                               + "...");
        }

        try {
            URL url = loc.toURL();
            PluginProperties props = null;
            JarFile jarFile = new JarFile(loc);
            ZipEntry entry = jarFile.getEntry("plugin.cfg");
            if (entry != null) {
                InputStream stream = jarFile.getInputStream(entry);
                props = new PluginProperties(url, stream);
                if (PluginProperties.debug()) {
                    System.out.println("zip entry " + entry + " found in "
                                       + jarFile.getName());
                }
            } else {
                if (PluginProperties.debug()) {
                    System.out.println("no zip entry found in "
                                       + jarFile.getName());
                }
            }
            return props;
        } catch (MalformedURLException e) {
            System.out.println("PluginJarLocationFinder: could not create URL from "
                               + loc);
        } catch (Exception e) {
            System.out.println("PluginJarLocationFinder: " + e);
        }
        return null;
    }

    public static void main(String[] args) {
        PluginProperties.getUserProperties().setProperty("debug", "false");
        PluginJarLocationFinder finder = new PluginJarLocationFinder(null);
        Collection found = finder.findPluginLocations();
        System.out.println("found the following locations:");
        System.out.println(CollectionLister.toString(found));
    }
}