/*
 * @author Timo Carl
 * Created: 06.05.2004
 *
 */
package de.renew.application;

import de.renew.plugin.PluginProperties;

import de.renew.shadow.ShadowNet;
import de.renew.shadow.ShadowNetSystem;
import de.renew.shadow.SyntaxException;

import java.io.IOException;
import java.io.ObjectInputStream;

import java.net.URL;

import java.util.Enumeration;
import java.util.Iterator;
import java.util.jar.JarFile;
import java.util.zip.ZipEntry;


/**
 * Special Extension that allows a plugin to add its Nets to a running Simulation.
 * Nets are loaded from the resources of the plugin.
 **/
public class AddPluginNetsExtension extends SimulatorExtensionAdapter {
    private JarFile jarRes = null;

    /**
     * Creates the extension.
     * @param resPath the <code>URL</code> to the <code>.jar</code> file
     *                where the nets reside.
     */
    public AddPluginNetsExtension(URL resPath) {
        if (PluginProperties.debug()) {
            System.out.println("AddPluginNetsExtension with URL " + resPath);
        }
        try {
            jarRes = new JarFile(resPath.getFile());
        } catch (IOException e) {
            System.out.println("Ressource path " + resPath
                               + "is not a valid jar file.");
        }
    }

    public void simulationSetup(SimulationEnvironment env) {
        //assumes that jarRes is valid because Plugin would not be functional.
        Enumeration entries = jarRes.entries();
        SimulatorPlugin sp = SimulatorPlugin.getCurrent();
        String name = null;
        ShadowNetSystem sns = null;
        while (entries.hasMoreElements()) {
            try {
                ZipEntry element = (ZipEntry) entries.nextElement();
                name = element.toString();
                if (!element.getName().endsWith(".sns")) {
                    continue;
                }
                Object o = new ObjectInputStream(jarRes.getInputStream(element))
                           .readObject();
                if (!(o instanceof ShadowNetSystem)) {
                    continue;
                }
                if (PluginProperties.debug()) {
                    System.out.println("adding net " + name);
                }
                if (sns == null) {
                    sns = (ShadowNetSystem) o;
                } else {
                    ShadowNetSystem newSns = (ShadowNetSystem) o;
                    for (Iterator iter = newSns.elements().iterator();
                                 iter.hasNext();) {
                        ShadowNet net = (ShadowNet) iter.next();
                        net.switchNetSystem(sns);
                    }
                }
            } catch (ClassNotFoundException ce) {
                System.out.println("This should not happen:" + ce
                                   + "\nAborting.");
                return;
            } catch (IOException ioe) {
                System.out.println("Resource" + name
                                   + "is not loadable.\nReason: " + ioe);
            }
        }

        try {
            sp.insertNets(sns);
        } catch (SyntaxException e) {
            System.out.println("Plugin nets contain a syntax error:\n"
                               + e.getMessage());
            System.out.println("Involved Objects: " + e.errorObjects);
            e.printStackTrace();
        }
    }
}