package de.renew.engine.common;

import de.renew.engine.searcher.LateExecutable;
import de.renew.engine.searcher.Tracer;


/**
 * I am a special executable. I will output a trace message
 * during execution. I am useful for debugging purposes
 * and for console traces of firing sequences.
 **/
public class TraceExecutable implements LateExecutable {

    /**
     * The message that I must output.
     **/
    private String message;

    /**
     * I (a trace executable) am created. I store the argument
     * message for further procession during the execute phase.
     *
     * @param message
     *   the message that I must output
     **/
    public TraceExecutable(String message) {
        this.message = message;
    }

    /**
     * I will always act during the phase 0. Maybe at some
     * time this could be configured.
     *
     * @return 0 (the phase that I must await)
     **/
    public int phase() {
        return TRACE;
    }

    /**
     * I output my message using the tracer provided as an
     * argument.
     *
     * @param tracer the tracer that want to receive my message.
     **/
    public void execute(Tracer tracer) {
        tracer.trace(message);
    }

    public void executeAfterException(Tracer tracer, Throwable t) {
    }

    /**
     * I cannot produce side effects that require the attention of the
     * simulation engine.
     *
     * @return false
     **/
    public boolean isLong() {
        return false;
    }
}