package de.renew.net.arc;

import de.renew.engine.searcher.EarlyExecutable;
import de.renew.engine.searcher.Tracer;
import de.renew.engine.searchqueue.SearchQueue;

import de.renew.net.PlaceInstance;

import de.renew.unify.Impossible;

import java.util.Vector;


class FlexibleInArcExecutable implements EarlyExecutable {
    PlaceInstance placeInstance;
    Vector tokens;
    FlexibleArc arc;
    int numRemoved;
    Vector removedTimes;

    FlexibleInArcExecutable(PlaceInstance placeInstance, Vector tokens,
                            FlexibleArc arc) {
        this.placeInstance = placeInstance;
        this.tokens = tokens;
        this.arc = arc;
        numRemoved = 0;
        removedTimes = new Vector();
    }

    public long lockPriority() {
        return placeInstance.lockOrder;
    }

    public int phase() {
        return INPUT;
    }

    public void lock() {
        placeInstance.lock.lock();
    }

    private void removeSingleToken(Object token) throws Impossible {
        double removedTime = placeInstance.removeToken(token, 0);
        removedTimes.addElement(new Double(removedTime));
        numRemoved++;
    }

    public void verify() throws Impossible {
        try {
            for (int i = 0; i < tokens.size(); i++) {
                Object token = tokens.elementAt(i);
                double removedTime = placeInstance.removeToken(token, 0);
                removedTimes.addElement(new Double(removedTime));
                numRemoved++;
            }
        } catch (Exception e) {
            // Undo all reservations.
            rollback();

            throw new Impossible();
        }
    }

    public void execute(Tracer tracer) {
        for (int i = 0; i < numRemoved; i++) {
            Object token = tokens.elementAt(i);
            if (arc.arcType == FlexibleArc.fastBoth) {
                if (arc.trace) {
                    InputArcExecutable.traceInArc(tracer, true, token,
                                                  placeInstance);
                }
                placeInstance.insertToken(token, SearchQueue.getTime());
            } else {
                if (arc.trace) {
                    InputArcExecutable.traceInArc(tracer, false, token,
                                                  placeInstance);
                }
            }
        }
    }

    public void rollback() {
        // We have to undo the previous removals. We cannot do this
        // without notifying the observers and listeners, because
        // it was not done silently. However, the database must not log
        // this modification.
        for (int i = 0; i < numRemoved; i++) {
            placeInstance.internallyInsertToken(tokens.elementAt(i),
                                                ((Double) removedTimes
                                                 .elementAt(i)).doubleValue(),
                                                false);
        }
    }

    public void unlock() {
        placeInstance.lock.unlock();
    }
}