package de.renew.remote;

import java.rmi.Remote;
import java.rmi.RemoteException;


/**
 * The tracer accessor encapsulates a tracer to provide
 * its functionality remotely. Use the tracer accessor
 * implementation to pass a tracer to the server.
 * @author Thomas Jacob
 */
public interface TracerAccessor extends Remote {

    /**
     * Process a trace message.
     * @param message The message to be output
     * @exception RemoteException If any RMI problem occurs.
     */
    public void trace(String message) throws RemoteException;

    /**
     * This tells me that all trace messages have arrived.
     * My descendents will perform an appropriate action, if any.
     * @exception RemoteException If any RMI problem occurs.
     */
    public void complete() throws RemoteException;
}