package de.renew.unify;

import de.renew.util.Types;
import de.renew.util.Value;

import java.util.Set;


public class TypeConstrainer implements Referer {
    final Class type;
    final private Reference reference;

    private TypeConstrainer(Class type, Object value, StateRecorder recorder)
            throws Impossible {
        this.type = type;
        reference = new Reference(value, this, recorder);
    }

    public static void constrain(Class type, Object value,
                                 StateRecorder recorder)
            throws Impossible {
        if (type == Types.UNTYPED) {
            // Nothing to do.
            return;
        } else if (Unify.isComplete(value)) {
            // Check value now, no need to create a type constrainer.
            check(type, value);
        } else {
            new TypeConstrainer(type, value, recorder);
        }
    }

    private static void check(Class type, Object value)
            throws Impossible {
        if (value instanceof Calculator) {
            Class valueType = ((Calculator) value).getType();

            if (type != valueType) {
                if (type.isPrimitive()
                            || !Types.allowsReferenceWidening(valueType, type)) {
                    throw new Impossible();
                }
            }
        } else if (value instanceof Value) {
            if (!type.isPrimitive()
                        || !Types.objectify(type).isInstance(((Value) value).value)) {
                throw new Impossible();
            }
        } else if (value == null) {
            if (type.isPrimitive()) {
                throw new Impossible();
            }
        } else if (!type.isInstance(value)) {
            System.err.println("Type error: Class " + type + ", object: "
                               + value);
            throw new Impossible();
        }
    }

    public void possiblyCompleted(Set listeners, StateRecorder recorder)
            throws Impossible {
        // I own only one reference, therefore that reference must have become
        // completed.
        check(type, reference.value);
    }
}