package de.renew.util;

import java.util.AbstractSet;
import java.util.Iterator;
import java.util.NoSuchElementException;


public final class IntegerRangeSet extends AbstractSet {
    private final int first;
    private final int last;

    public IntegerRangeSet(int first, int last) {
        this.first = first;
        this.last = last;
    }

    public int size() {
        return last - first + 1;
    }

    public boolean contains(Object o) {
        if (o instanceof Integer) {
            int i = ((Integer) o).intValue();
            return first <= i && i <= last;
        }
        return false;
    }

    public Iterator iterator() {
        return new Iterator() {
                private int next = first;
                private boolean hasNext = true;

                public void remove() {
                    throw new UnsupportedOperationException();
                }

                public boolean hasNext() {
                    return hasNext;
                }

                public Object next() {
                    if (!hasNext) {
                        throw new NoSuchElementException();
                    }
                    hasNext = (next != last);
                    return new Integer(next++);
                }
            };
    }
}